/*
 * Copyright (c) 2022 Proton AG
 *
 * This file is part of Proton Mail.
 *
 * Proton Mail is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Proton Mail is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Proton Mail. If not, see https://www.gnu.org/licenses/.
 */

package ch.protonmail.android.uitests.robots.settings

import androidx.annotation.IdRes
import androidx.appcompat.widget.SwitchCompat
import ch.protonmail.android.uitests.robots.settings.SettingsMatchers.withSettingsHeader
import ch.protonmail.android.uitests.robots.settings.account.privacy.PrivacySettingsRobot
import me.proton.fusion.Fusion

object SettingsActions : Fusion {

    fun changeToggleState(state: Boolean, tag: String, @IdRes switch: SwitchCompat) {
        val currentSwitchState = switch.isChecked

        when (state xor currentSwitchState) {
            true -> {
                clickSwitchToggleView(tag, switch.id)
            }
            false -> {
                clickSwitchToggleView(tag, switch.id)
                clickSwitchToggleView(tag, switch.id)
            }
        }
    }

    private fun clickSwitchToggleView(title: String, @IdRes switchId: Int) {
        recyclerView
            .withId(PrivacySettingsRobot.settingsRecyclerView)
            .onHolderItem(withSettingsHeader(title))
            .onItemChildView(view.withId(switchId))
            .click()
    }
}
